/* -*-c-*-
 *
 * $Id: quine.h 2187 2004-09-04 07:50:08Z mdw $
 *
 * Definitions for building quines
 *
 * (c) 1999 Mark Wooding
 */

/*----- Licensing notice --------------------------------------------------*
 *
 * This file is part of Quine
 *
 * Quine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Quine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Quine; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef QUINE_H
#define QUINE_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Required headers --------------------------------------------------*/

#include <stdio.h>

/*----- External tables ---------------------------------------------------*/

extern const char **qq_table[];
extern const char *qq_qqlib[];

/*----- Functions provided ------------------------------------------------*/

/* --- @qq_xlate@ --- *
 *
 * Arguments:	@FILE *fp@ = output file handle
 *		@const char *p@ = pointer to encoded text
 *
 * Returns:	---
 *
 * Use:		Writes the decoded string to the given file handle.
 */

extern void qq_xlate(FILE */*fp*/, const char */*p*/);

/* --- @qq_file@ --- *
 *
 * Arguments:	@FILE *fp@ = output file handle
 *		@const char **p@ = pointer to the output array
 *
 * Returns:	---
 *
 * Use:		Writes the contents of a file to the output.
 */

extern void qq_file(FILE */*fp*/, const char **/*p*/);

/* --- @qq_head@ --- *
 *
 * Arguments:	@FILE *fp@ = output file handle
 *
 * Returns:	---
 *
 * Use:		Writes the head of a `qqout.c' file.
 */

extern void qq_head(FILE */*fp*/);

/* --- @qq_body@ --- *
 *
 * Arguments:	@FILE *fp@ = output file handle
 *		@const char ***p@ = pointer to main table
 *
 * Returns:	---
 *
 * Use:		Writes the body table of a `qqout.c' file.
 */

extern void qq_body(FILE */*fp*/, const char ***/*p*/);

/* --- @qq_tail@ --- *
 *
 * Arguments:	@FILE *fp@ = output file handle
 *		@const char **qql@ = pointer to qqlib file array
 *		@size_t fno@ = number of files written
 *		@const char *fn@ = name of `qqout.c' file
 *
 * Returns:	---
 *
 * Use:		Writes the head of a `qqout.c' file.
 */

extern void qq_tail(FILE */*fp*/, const char **/*qql*/,
		    size_t /*fno*/, const char */*fn*/);

/* --- @qq_mkfile@ --- *
 *
 * Arguments:	@const char *fn@ = pointer to a filename
 *		@int mode@ = mode to create file with
 *
 * Returns:	A handle for the created file.
 *
 * Use:		Creates a file, and leading directories and stuff.
 */

#ifdef QUINE_PORTABLE
extern FILE *qq_mkfile(const char */*fn*/);
#else
extern FILE *qq_mkfile(const char */*fn*/, int /*mode*/);
#endif

/* --- @qq_dump@ --- *
 *
 * Arguments:	@FILE *fp@ = stream to dump on
 *
 * Returns:	---
 *
 * Use:		Writes the program's source code to the given file.
 */

extern void qq_dump(FILE */*fp*/);

/* --- @qq_create@ --- *
 *
 * Arguments:	---
 *
 * Returns:	---
 *
 * Use:		Writes a source distribution to the current directory.
 */

extern int qq_create(void);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
